@@ -3,6 +3,7 @@ import re
 import py
 from pkg_resources import parse_version, Requirement
 from .types import CompareMixin
+from .types import cached_property
 from .validation import normalize_name
 
 
@@ -122,11 +123,17 @@ def splitext_archive(basename):
 class Version(CompareMixin):
     def __init__(self, versionstring):
         self.string = versionstring
-        self.cmpval = parse_version(versionstring)
+
+    @cached_property
+    def cmpval(self):
+        return parse_version(self.string)
 
     def __str__(self):
         return self.string
 
+    def __repr__(self):
+        return "Version(%r)" % self.string
+
     def is_prerelease(self):
         if hasattr(self.cmpval, 'is_prerelease'):
             return self.cmpval.is_prerelease
